@@ -10,7 +10,7 @@ import pandas as pd
 from ipyevents import Event
 from ipywidgets import Widget, Layout
 
-__version__ = "0.7.5"
+__version__ = "0.7.6"
 
 
 class SAMGUI(object):
@@ -783,6 +783,7 @@ class SAMGUI(object):
         self.rb_dict['L3']=l3
         self.rb_dict['L4']=l4
         self.rb_dict['NNA']=nna
+        self.rb_dict['KNN']=knn
         self.rb_dict['NORM']=norm
         self.rb_dict['DISTANCE']=distance
         self.rb_dict['PROJ']=proj
@@ -1062,6 +1063,7 @@ class SAMGUI(object):
         log = widgets.Checkbox(
             indent=False, value=False, description="Log cbar", layout={"width": "20%"}
         )
+
         show_colorbar = widgets.Checkbox(
             indent=False, value=False, description="Show cbar", layout={"width": "20%"}
         )
@@ -1327,7 +1329,7 @@ class SAMGUI(object):
 
                 if self.cs_dict['AVG'].value:
                     if 'X_knn_avg' not in s.adata.layers.keys():
-                        s.dispersion_ranking_NN();
+                        s.dispersion_ranking_NN(save_avgs=True);
 
                     x = s.adata[:, gene].layers["X_knn_avg"]
                     if sp.issparse(x):
@@ -1335,37 +1337,12 @@ class SAMGUI(object):
                     else:
                         a = x.flatten()
                 else:
-                    x = s.adata_raw[:, gene][s.adata.obs_names, :].X
+                    x = s.adata[:, gene][s.adata.obs_names, :].X
                     if sp.issparse(x):
                         a = x.A.flatten()
                     else:
                         a = x.flatten()
 
-                if self.cs_dict['AVG'].value:
-                    if a.sum() == 0:
-                        x = s.adata_raw[:, gene][s.adata.obs_names, :].X
-                        if sp.issparse(x):
-                            a = x.A.flatten()
-                        else:
-                            a = x.flatten()
-
-                        norm = self.preprocess_args.get("norm", "log")
-                        if norm is not None:
-                            if norm.lower() == "log":
-                                a = np.log2(a + 1)
-                            elif norm.lower() == "ftt":
-                                a = np.sqrt(a) + np.sqrt(a + 1) - 1
-                            elif norm.lower() == "asin":
-                                a = np.arcsinh(a)
-                else:
-                    norm = self.preprocess_args.get("norm", "log")
-                    if norm is not None:
-                        if norm.lower() == "log":
-                            a = np.log2(a + 1)
-                        elif norm.lower() == "ftt":
-                            a = np.sqrt(a) + np.sqrt(a + 1) - 1
-                        elif norm.lower() == "asin":
-                            a = np.arcsinh(a)
 
                 if self.GENE_KEY != "":
                     title = gene + "; " + str(s.adata.var[self.GENE_KEY].T[gene])
@@ -1589,7 +1566,7 @@ class SAMGUI(object):
         title = self.cs_dict['DAM'].value.split("_clusters")[0]
 
         if issubclass(labels.dtype.type, np.number):
-            if issubclass(labels.dtype.type, np.float) or nlabels > 300:
+            if issubclass(labels.dtype.type, np.float):
                 self.update_colors_expr(labels, title)
                 return
 
@@ -1606,10 +1583,8 @@ class SAMGUI(object):
             x = cl.interp(x, int(nlabels))
 
         lbls, inv = np.unique(labels, return_inverse=True)
-
-        dd = self.cs_dict['ACC']
         self.dd_opts[self.stab.selected_index] = ["Toggle cluster"] + list(lbls)
-        dd.options = self.dd_opts[self.stab.selected_index]
+        self.cs_dict['ACC'].options = self.dd_opts[self.stab.selected_index]
 
         if issubclass(labels.dtype.type, np.character):
             tickvals = np.arange(lbls.size)
@@ -1760,11 +1735,10 @@ class SAMGUI(object):
         }
 
     def pick_cells_dd(self, txt):
-        if txt["new"] != "":
-            al = txt["new"]
-
+        if txt["new"] != "Toggle cluster":
+            al = str(txt["new"])
             sel = self.selected[self.stab.selected_index]
-            als = self.active_labels[self.stab.selected_index]
+            als = self.active_labels[self.stab.selected_index].astype('str')
             ratio = sel[als == al].sum() / sel[als == al].size
             if ratio >= 0.5:
                 sel[als == al] = False
